"""

.. _reset-password:

reset_password
==============

it's a script whose goal is to be used in scripts (jmb-sync) when creating
local db on developer's personal pc.

``jmb-sync`` resets the admin password so that browser can prefill password

::

  dj reset_password user pass


"""
from __future__ import unicode_literals

import getpass
import logging

try:
    from logutils.colorize import ColorizingStreamHandler as StreamHandler
except ImportError:
    from logging import StreamHandler


from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand, CommandError
from django.db import DEFAULT_DB_ALIAS

logger = logging.getLogger('jmb')

class Command(BaseCommand):

    help = """reset_password user pass: change a user's password for django.contrib.auth.
    Pass password in command line"""

    #requires_model_validation = False
    requires_system_checks = False

    def add_arguments(self, parser):
        parser.add_argument('username', help='the username we want to change the password for')
        parser.add_argument('password', help='the password we want to set (optional)',
                            default=None, nargs="?")
        # -q changes opt --verbose setting to const
        parser.add_argument("-q", "--quiet", action="store_false", dest="console")
        parser.add_argument('-l', '--debug-level', default='info',
                    help='Set debug level (debug, info, warnings) for console', )


    def handle(self, *args, **options):

        username = options['username']
        password = options['password']

        if not password:
            password = getpass.getuser()

        UserModel = get_user_model()

        level = getattr(logging,  options.get('debug_level').upper())
        if options.get('console'):
            console_handler = StreamHandler()
            console_handler.setLevel(level)
            logger.addHandler(console_handler)
            logger.setLevel(level)

        try:
            u = UserModel._default_manager.using(options.get('database')).get(**{
                UserModel.USERNAME_FIELD: username
            })
        except UserModel.DoesNotExist:
            raise CommandError("user '%s' does not exist" % username)

        logger.info("Changing password for user '%s'" % u)

        u.set_password(password)
        u.save()
        logger.info("Password changed successfully for user '%s'" % u)
        return
